// Copyright © 2022-2025 Gerald B. Cox
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

package beets

import (
	"encoding/json"
	"errors"
	"fmt"
	"io"
	"log/slog" // New: Import for structured logging
	"net/http"
	"net/url"
	"os"
	"strings"
	"time"

	lb "codeberg.org/gbcox/beetbrainz/listenbrainz"
	"codeberg.org/gbcox/beetbrainz/types"
)

type BeetsData struct {
	Title          string `json:"title"`
	Album          string `json:"album"`
	Artist         string `json:"artist"`
	RecordingId    string `json:"mb_trackid"`
	ReleaseId      string `json:"mb_albumid"`
	ArtistId       string `json:"mb_artistid"`
	ReleaseGroupId string `json:"mb_releasegroupid"`
	WorkId         string `json:"mb_workid"`
}

func (data *BeetsData) String() string {
	return fmt.Sprintf("%s - %s (%s)", data.Artist, data.Title, data.Album)
}

func GetMetadataForItem(item *types.MediaItem) (*lb.TrackMetadata, error) {
	tm := lb.TrackMetadata{
		AdditionalInfo: &lb.AdditionalInfo{
			ListeningFrom: "Beetbrainz Media Server",
		},
		ArtistName:  item.Artist,
		ReleaseName: item.Album,
		TrackName:   item.Track,
	}

	var beetsData *BeetsData
	if os.Getenv("BEETS_IP") != "" {
		slog.Info("Getting additional track metadata from Beets", "item", item.String())
		var err error
		beetsData, err = matchWithBeets(item)
		if err != nil {
			slog.Warn("Could not match with Beets", "error", err, "item", item.String())
			// We continue here even if Beets lookup fails, as original metadata might still be usable.
		}
	} else {
		slog.Info("Beets IP not configured, skipping Beets lookup.")
	}


	if beetsData != nil {
		slog.Info("Adding Beets data to ListenBrainz submission", "beets_item", beetsData.String())
		tm.AdditionalInfo.ReleaseMbid = beetsData.ReleaseId
		tm.AdditionalInfo.ArtistMbids = []string{beetsData.ArtistId}
		tm.AdditionalInfo.RecordingMbid = beetsData.RecordingId

		// Overwrite with Beets' more accurate data if available
		tm.ArtistName = beetsData.Artist
		tm.ReleaseName = beetsData.Album
		tm.TrackName = beetsData.Title
	} else {
		slog.Info("No Beets data found for item, proceeding with original metadata (if available)", "item", item.String())
	}

	return &tm, nil
}

func matchWithBeets(item *types.MediaItem) (*BeetsData, error) {
	beetsResults, err := getBeetsData(item.Track)
	if err != nil {
		return nil, fmt.Errorf("failed to retrieve data from Beets for '%s': %w", item.Track, err)
	}

	if len(beetsResults) == 0 {
		slog.Info("No Beets data received for item query", "item", item.String())
		return nil, errors.New("no Beets data received for query")
	}

	// Try to find a match
	matchedBeetsData := findBestMatch(beetsResults, item)

	if matchedBeetsData != nil {
		slog.Info("Item exactly matches with Beets data", "webhook_item", item.String(), "beets_match", matchedBeetsData.String())
		return matchedBeetsData, nil
	} else {
		// Only log all results if no suitable match was found
		var resultStrings []string
		for _, br := range beetsResults {
			resultStrings = append(resultStrings, br.String())
		}
		slog.Info("No suitable Beets match found", "webhook_item", item.String(), "beets_results_count", len(beetsResults), "beets_all_results", strings.Join(resultStrings, " || "))
		return nil, errors.New("no suitable match for item in Beets DB")
	}
}

func getBeetsData(title string) ([]*BeetsData, error) {
	beetsIP := os.Getenv("BEETS_IP")
	if beetsIP == "" {
		return nil, errors.New("BEETS_IP environment variable not set")
	}
	beetsPort := os.Getenv("BEETS_PORT")
	if beetsPort == "" {
		beetsPort = "8337" // Default port
	}

	escapedTitle := strings.ReplaceAll(title, "/", "\\")
	beetsURL := "http://" + beetsIP + ":" + beetsPort + "/item/query/title:" + url.PathEscape(escapedTitle)

	client := &http.Client{
		Timeout: 10 * time.Second,
	}

	resp, err := client.Get(beetsURL)
	if err != nil {
		slog.Error("Beets request failed", "error", err, "url", beetsURL)
		return nil, fmt.Errorf("beets request failed: %w", err)
	}
	defer resp.Body.Close()

	if resp.StatusCode != http.StatusOK {
		bodyBytes, _ := io.ReadAll(resp.Body)
		slog.Error("Beets API returned non-200 status code", "status_code", resp.StatusCode, "query_title", title, "response_body", string(bodyBytes))
		return nil, fmt.Errorf("Beets API returned unexpected status code %d for '%s': %s", resp.StatusCode, title, string(bodyBytes))
	}

	var rj map[string][]*BeetsData
	err = json.NewDecoder(resp.Body).Decode(&rj)
	if err != nil {
		slog.Error("Failed to decode the response from Beets", "error", err, "query_title", title)
		return nil, fmt.Errorf("failed to decode Beets response: %w", err)
	}

	return rj["results"], nil
}

// findBestMatch extracts the matching logic into a separate function,
// allowing matchWithBeets to decide on logging based on its return.
func findBestMatch(beetsResults []*BeetsData, refItem *types.MediaItem) *BeetsData {
	// Prioritize exact matches (case-insensitive for robustness)
	for _, bd := range beetsResults {
		if strings.EqualFold(refItem.Track, bd.Title) &&
			strings.EqualFold(refItem.Album, bd.Album) &&
			strings.EqualFold(refItem.Artist, bd.Artist) {
			return bd // Found exact match
		}
	}

	// Then try partial matches (e.g., "Various Artists" or almostEqual artist)
	for _, bd := range beetsResults {
		if strings.EqualFold(refItem.Track, bd.Title) && strings.EqualFold(refItem.Album, bd.Album) {
			if strings.EqualFold(refItem.Artist, "Various Artists") || almostEqual(bd.Artist, refItem.Artist) {
				return bd // Found partial match
			}
		}
	}

	// No suitable match found
	return nil
}

func almostEqual(a, b string) bool {
	// Simple contains check, converts both to lower for case-insensitivity
	lowerA := strings.ToLower(a)
	lowerB := strings.ToLower(b)
	return strings.Contains(lowerA, lowerB) || strings.Contains(lowerB, lowerA)
}
