// Copyright © 2022-2025 Gerald B. Cox
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

package common

import (
	"fmt" // Added back for fmt.Sprintf
	"log/slog"
	"net/http" // Added back for http.Client
	"os"         // Added back for os.Getenv
	"strings"    // Added back for strings.ReplaceAll
	"time"       // Added back for http.Client timeout

	"codeberg.org/gbcox/beetbrainz/beets"
	env "codeberg.org/gbcox/beetbrainz/environ" // This import is now correctly used
	lb "codeberg.org/gbcox/beetbrainz/listenbrainz"
	"codeberg.org/gbcox/beetbrainz/types"
)

// Request defines the common structure for all incoming webhook requests.
type Request struct {
	Event     string
	MediaType string
	User      string
	Item      *types.MediaItem
	Metadata  *lb.TrackMetadata // This field is not directly used from input, but for internal passing
}

// Define generic event constants that media server handlers can use.
const (
	EventPlaybackStart = "start" // A normalized event for playback initiation
	EventPlaybackStop  = "stop"  // A normalized event for playback cessation
)

// Define constants for ListenBrainz actions derived from events
const (
	LBACTION_PLAYING_NOW = "playing_now" // Corrected this line
	LBACTION_SCROBBLE    = "scrobble"    // Maps to ListenBrainz "submit listens" API
	LBACTION_IGNORE      = "ignore"      // For events we explicitly want to ignore
)

// eventActionMap maps specific webhook event strings (from media servers OR normalized)
// to their corresponding ListenBrainz action.
var eventActionMap = map[string]string{
	// --- Emby Events ---
	"playback.start":    LBACTION_PLAYING_NOW,
	"playback.unpause":  LBACTION_PLAYING_NOW,
	"playback.stop":     LBACTION_SCROBBLE,    // Emby's "playback.stop"
	"playback.scrobble": LBACTION_SCROBBLE,
	"item.markplayed":   LBACTION_SCROBBLE,

	// --- Plex Events ---
	"media.play":     LBACTION_PLAYING_NOW,
	"media.resume":   LBACTION_PLAYING_NOW,
	"media.scrobble": LBACTION_SCROBBLE,
	"media.stop":     LBACTION_SCROBBLE, // Plex's "media.stop"

	// --- Tautulli Events (proxied from Plex/Emby/Jellyfin) ---
	"play":     LBACTION_PLAYING_NOW,
	"resume":   LBACTION_PLAYING_NOW,
	"scrobble": LBACTION_SCROBBLE,
	// "stop":     LBACTION_SCROBBLE, // REMOVED: This key now conflicts with EventPlaybackStop
	"pause": LBACTION_IGNORE,

	// --- Normalized Events (from jellyfin.go and potentially other handlers) ---
	// This entry now covers "stop" events from Tautulli as well if Tautulli also sends "stop"
	EventPlaybackStart: LBACTION_PLAYING_NOW, // Maps "start" (from jellyfin.go) to "playing_now"
	EventPlaybackStop:  LBACTION_SCROBBLE,    // Maps "stop" (from jellyfin.go or Tautulli) to "scrobble"
}

// getBeetsAPIURL constructs the Beets API URL from environment variables.
// Added back as it was present in previous correct versions of common.go.
func getBeetsAPIURL() string {
	beetsIP := os.Getenv("BEETS_IP")
	if beetsIP == "" {
		return "" // Beets integration disabled
	}
	beetsPort := os.Getenv("BEETS_PORT")
	if beetsPort == "" {
		beetsPort = "8337" // Default Beets port
	}
	return fmt.Sprintf("http://%s:%s/item/query", beetsIP, beetsPort)
}

// fetchBeetsMetadata queries the Beets web API for additional metadata.
// Added back as it was present in previous correct versions of common.go.
func fetchBeetsMetadata(item *types.MediaItem) {
	beetsAPIURL := getBeetsAPIURL()
	if beetsAPIURL == "" {
		slog.Debug("Beets API URL not configured, skipping metadata enrichment.")
		return
	}

	query := fmt.Sprintf("artist:%s album:%s title:%s", item.Artist, item.Album, item.Track)
	reqURL := fmt.Sprintf("%s?q=%s", beetsAPIURL, strings.ReplaceAll(query, " ", "+"))
	slog.Debug("Querying Beets for metadata", "url", reqURL)

	client := http.Client{Timeout: 5 * time.Second}
	resp, err := client.Get(reqURL)
	if err != nil {
		slog.Error("Failed to query Beets API", "error", err, "query", query)
		return
	}
	defer resp.Body.Close()

	if resp.StatusCode != http.StatusOK {
		slog.Warn("Beets API returned non-OK status", "status", resp.Status, "query", query)
		return
	}

	slog.Info("Successfully queried Beets API (parsing response not implemented yet)", "query", query)
}

// HandleRequest processes a generic webhook request from a media server.
func HandleRequest(r *Request) {
	// Step 1: Authenticate user and get API token FIRST
	apiToken := env.GetApiToken(r.User)
	if apiToken == "" {
		slog.Warn("No API token configured for user", "user", r.User, "event", r.Event, "item", r.Item.String())
		return
	}

	// Step 2: Determine ListenBrainz action based on event and media type
	lbAction := eventActionMap[r.Event]
	if lbAction == "" {
		slog.Info("Unknown event received, ignoring", "event", r.Event, "media_type", r.MediaType, "user", r.User, "item", r.Item.String())
		return
	}

	if lbAction == LBACTION_IGNORE {
		slog.Info("Event mapped to ignore action, skipping", "event", r.Event, "media_type", r.MediaType, "user", r.User, "item", r.Item.String())
		return
	}

	// Accept both "track" (from Plex/Tautulli) and "audio" (from Emby/Jellyfin) as valid media types
	if r.MediaType != "track" && r.MediaType != "audio" {
		slog.Info("Non-track/audio media type received, ignoring", "event", r.Event, "media_type", r.MediaType, "user", r.User, "item", r.Item.String())
		return
	}

	slog.Info("Processing request",
		"event", r.Event,
		"lb_action", lbAction,
		"user", r.User,
		"item", r.Item.String(),
		"media_type", r.MediaType,
	)

	// Step 3: Get metadata from Beets
	tm, err := beets.GetMetadataForItem(r.Item)
	if err != nil {
		slog.Error("Failed to get Beets metadata for item", "error", err, "item", r.Item.String(), "user", r.User)
		// We still attempt to scrobble with available metadata even if Beets fails
	}

	// Step 4: Send scrobble to ListenBrainz based on determined action
	switch lbAction {
	case LBACTION_PLAYING_NOW:
		err := lb.PlayingNow(apiToken, tm)
		if err != nil {
			slog.Error("Playing now request failed", "error", err, "item", r.Item.String(), "user", r.User)
		} else {
			slog.Info("User is playing now",
				"user", r.User,
				"artist", tm.ArtistName,
				"track", tm.TrackName,
				"album", tm.ReleaseName,
			)
		}
	case LBACTION_SCROBBLE:
		err := lb.SubmitListen(apiToken, tm)
		if err != nil {
			slog.Error("Scrobble request failed", "error", err, "item", r.Item.String(), "user", r.User)
		} else {
			slog.Info("User scrobbled",
				"user", r.User,
				"artist", tm.ArtistName,
				"track", tm.TrackName,
				"album", tm.ReleaseName,
			)
		}
	}
}
