// Copyright © 2022-2025 Gerald B. Cox
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

package emby

import (
	"encoding/json"
	"errors" // <--- ADDED THIS IMPORT
	"fmt"
	"log/slog" // Already here for structured logging
	"net/http"
	"strings"

	"codeberg.org/gbcox/beetbrainz/common"
	"codeberg.org/gbcox/beetbrainz/types"
)

type EmbyItem struct {
	Type        string `json:"Type"`
	Track       string `json:"Name"`
	Album       string `json:"Album"`
	Artist      string `json:"AlbumArtist"` // Emby sends AlbumArtist
}

func (item *EmbyItem) String() string {
	return fmt.Sprintf("%s - %s (%s)", item.Artist, item.Track, item.Album)
}

func (item *EmbyItem) AsMediaItem() *types.MediaItem {
	return &types.MediaItem{
		Artist: item.Artist,
		Album:  item.Album,
		Track:  item.Track,
	}
}

type EmbyAccount struct {
	Name string `json:"Name"` // Emby sends user name in Account.Name
}

type EmbyRequest struct {
	Event   string      `json:"Event"`
	Account EmbyAccount `json:"User"` // Note: "User" in JSON for account info
	Item    EmbyItem    `json:"Item"`
}

func HandleRequest(w http.ResponseWriter, r *http.Request) {
	if r.Method != "POST" {
		slog.Warn("Emby webhook received unsupported method", "method", r.Method, "expected_method", "POST")
		http.Error(w, "Method Not Allowed", http.StatusMethodNotAllowed)
		return
	}

	rq, err := parseRequest(r)
	if err != nil {
		slog.Error("Failed to parse Emby webhook request", "error", err)
		http.Error(w, fmt.Sprintf("Bad Request: %s", err.Error()), http.StatusBadRequest)
		return
	}

	common.HandleRequest(rq)

	w.WriteHeader(http.StatusOK)
}

func parseRequest(r *http.Request) (*common.Request, error) {
	var embyRequest EmbyRequest
	var err error

	contentType := r.Header.Get("Content-Type")

	if strings.Contains(contentType, "application/json") {
		err = json.NewDecoder(r.Body).Decode(&embyRequest)
		if err != nil {
			return nil, fmt.Errorf("failed to decode JSON from request body: %w", err)
		}
	} else if strings.Contains(contentType, "multipart/form-data") {
		err = r.ParseMultipartForm(16 << 20) // 16 MB limit for memory
		if err != nil {
			return nil, fmt.Errorf("failed to parse multipart form: %w", err)
		}
		data := []byte(r.FormValue("data"))
		err = json.Unmarshal(data, &embyRequest)
		if err != nil {
			return nil, fmt.Errorf("failed to unmarshal JSON from 'data' form field: %w", err)
		}
	} else {
		return nil, fmt.Errorf("unsupported Content-Type: %s", contentType)
	}

	// Basic validation of crucial fields
	if embyRequest.Event == "" {
		return nil, errors.New("Emby webhook payload missing 'Event' field")
	}
	if embyRequest.Item.Type != "Audio" { // Emby typically sends "Audio" for tracks
		return nil, fmt.Errorf("Emby webhook 'Item.Type' is '%s', only 'Audio' (track) is supported", embyRequest.Item.Type)
	}
	if embyRequest.Item.Track == "" || embyRequest.Item.Album == "" { // Artist can sometimes be empty, but track/album are critical
		return nil, errors.New("Emby webhook payload missing required music metadata (Item.Name or Item.Album)")
	}
	if embyRequest.Account.Name == "" { // Corrected from Account.Track
		return nil, errors.New("Emby webhook payload missing user 'User.Name'")
	}


	rq := &common.Request{
		Event:     embyRequest.Event,
		MediaType: strings.ToLower(embyRequest.Item.Type), // Normalize to "audio" or "track"
		User:      embyRequest.Account.Name, // Corrected from Account.Track
		Item:      embyRequest.Item.AsMediaItem(),
	}

	return rq, nil
}
