// Copyright © 2022-2025 Gerald B. Cox
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

package main

import (
	"fmt"
	"log"       // Still needed for log.Fatalf and log.Fatal(server.Serve(l))
	"log/slog"  // New: Import for structured logging
	"net"
	"net/http"
	"os"
	"strconv"
	"strings"
	"time"
	
	"codeberg.org/gbcox/beetbrainz/emby"
	"codeberg.org/gbcox/beetbrainz/plex"
	"codeberg.org/gbcox/beetbrainz/tautulli"
	"codeberg.org/gbcox/beetbrainz/jellyfin" // NEW: Import the jellyfin package
)

// These variables will be set at compile time by the Go linker (-ldflags).
// They hold information about the build version and Git commit.
var (
	appVersion = "dev"       // The application version (e.g., 1.0.0), overridden by ldflags
	commit     = "none"      // The full Git commit hash, overridden by ldflags
	shortCommit = "none"     // The short Git commit hash, overridden by ldflags
)

func main() {
	// --- Configure Slog ---
	// Output JSON logs to standard output.
	// This makes logs easily parsable by log aggregation systems.
	slog.SetDefault(slog.New(slog.NewJSONHandler(os.Stdout, nil)))

	// --- Validate BEETBRAINZ_IP and BEETBRAINZ_PORT ---
	addr := os.Getenv("BEETBRAINZ_IP")
	if addr == "" {
		addr = "0.0.0.0" // Default to listen on all interfaces
	}

	portStr := os.Getenv("BEETBRAINZ_PORT")
	if portStr == "" {
		portStr = "5000" // Default port
	}
	port, err := strconv.Atoi(portStr)
	if err != nil {
		log.Fatalf("ERROR: Invalid BEETBRAINZ_PORT '%s'. Must be a number: %v", portStr, err)
	}
	if port < 1 || port > 65535 {
		log.Fatalf("ERROR: BEETBRAINZ_PORT %d is out of valid range (1-65535).", port)
	}
	address := fmt.Sprintf("%s:%d", addr, port)

	// --- Validate BEETS_IP and BEETS_PORT (if Beets integration is enabled) ---
	beetsIP := os.Getenv("BEETS_IP")
	beetsPortStr := os.Getenv("BEETS_PORT")

	if beetsIP != "" { // Only validate Beets settings if BEETS_IP is provided
		if beetsPortStr == "" {
			beetsPortStr = "8337" // Default Beets port
		}
		beetsPort, err := strconv.Atoi(beetsPortStr)
		if err != nil {
			log.Fatalf("ERROR: Invalid BEETS_PORT '%s'. Must be a number: %v", beetsPortStr, err)
		}
		if beetsPort < 1 || beetsPort > 65535 {
			log.Fatalf("ERROR: BEETS_PORT %d is out of valid range (1-65535).", beetsPort)
		}
		slog.Info("Beets integration enabled", "ip", beetsIP, "port", beetsPort)
	} else {
		slog.Info("Beets IP not configured. Beets integration will be disabled.")
	}

	// --- Validate USER_TOKENS ---
	userTokens := os.Getenv("USER_TOKENS")
	if userTokens == "" {
		log.Fatalf("ERROR: USER_TOKENS environment variable is not set. This is required for ListenBrainz authentication.")
	}

	if !strings.Contains(userTokens, ":") {
		log.Fatalf("ERROR: USER_TOKENS environment variable is malformed. It must contain at least one 'user:token' pair separated by ':'. Current value: '%s'", userTokens)
	}

	slog.Info("USER_TOKENS environment variable is set and appears to be in a parseable format.")

	// --- Setup HTTP Server ---
	sm := http.NewServeMux()
	sm.HandleFunc("/plex", plex.HandleRequest)       // Existing Plex handler
	sm.HandleFunc("/tautulli", tautulli.HandleRequest) // Existing Tautulli handler
	sm.HandleFunc("/emby", emby.HandleRequest)       // Existing Emby handler
	sm.HandleFunc("/jellyfin", jellyfin.HandleRequest) // NEW: Jellyfin webhook handler

	// Create the HTTP server with timeouts
	server := &http.Server{
		Addr:    address,
		Handler: sm,
		ReadTimeout:  10 * time.Second,
		WriteTimeout: 10 * time.Second,
		IdleTimeout:  30 * time.Second,
	}

	l, err := net.Listen("tcp4", address)
	if err != nil {
		log.Fatalf("Failed to listen on %s: %v", address, err)
	}

	slog.Info("Beetbrainz server starting", "address", l.Addr().String())
	// log.Fatal is used here as it will exit the program if Serve returns an error
	log.Fatal(server.Serve(l))
}
